@@ -26,7 +26,15 @@ class OpenIdConnectAuth(BaseOAuth2):
     """
     Base class for Open ID Connect backends.
     Currently only the code response type is supported.
+
+    It can also be directly instantiated as a generic OIDC backend.
+    To use it you will need to set at minimum:
+
+    SOCIAL_AUTH_OIDC_OIDC_ENDPOINT = 'https://.....'  # endpoint without /.well-known/openid-configuration
+    SOCIAL_AUTH_OIDC_KEY = '<client_id>'
+    SOCIAL_AUTH_OIDC_SECRET = '<client_secret>'
     """
+    name = 'oidc'
     # Override OIDC_ENDPOINT in your subclass to enable autoconfig of OIDC
     OIDC_ENDPOINT = None
     ID_TOKEN_MAX_AGE = 600
@@ -37,46 +45,58 @@ class OpenIdConnectAuth(BaseOAuth2):
     REVOKE_TOKEN_METHOD = 'GET'
     ID_KEY = 'sub'
     USERNAME_KEY = 'preferred_username'
+    JWT_ALGORITHMS = ['RS256']
+    JWT_DECODE_OPTIONS = dict()
+    # When these options are unspecified, server will choose via openid autoconfiguration
     ID_TOKEN_ISSUER = ''
     ACCESS_TOKEN_URL = ''
     AUTHORIZATION_URL = ''
     REVOKE_TOKEN_URL = ''
     USERINFO_URL = ''
     JWKS_URI = ''
-    JWT_ALGORITHMS = ['RS256']
-    JWT_DECODE_OPTIONS = dict()
+    TOKEN_ENDPOINT_AUTH_METHOD = ''
 
     def __init__(self, *args, **kwargs):
         self.id_token = None
         super().__init__(*args, **kwargs)
 
     def authorization_url(self):
-        return self.AUTHORIZATION_URL or \
+        return self.setting('AUTHORIZATION_URL', self.AUTHORIZATION_URL) or \
             self.oidc_config().get('authorization_endpoint')
 
     def access_token_url(self):
-        return self.ACCESS_TOKEN_URL or \
+        return self.setting('ACCESS_TOKEN_URL', self.ACCESS_TOKEN_URL) or \
             self.oidc_config().get('token_endpoint')
 
     def revoke_token_url(self, token, uid):
-        return self.REVOKE_TOKEN_URL or \
+        return self.setting('REVOKE_TOKEN_URL', self.REVOKE_TOKEN_URL) or \
             self.oidc_config().get('revocation_endpoint')
 
     def id_token_issuer(self):
-        return self.ID_TOKEN_ISSUER or \
+        return self.setting('ID_TOKEN_ISSUER', self.ID_TOKEN_ISSUER) or \
             self.oidc_config().get('issuer')
 
     def userinfo_url(self):
-        return self.USERINFO_URL or \
+        return self.setting('USERINFO_URL', self.USERINFO_URL) or \
             self.oidc_config().get('userinfo_endpoint')
 
     def jwks_uri(self):
-        return self.JWKS_URI or \
+        return self.setting('JWKS_URI', self.JWKS_URI) or \
             self.oidc_config().get('jwks_uri')
 
+    def use_basic_auth(self):
+        method = self.setting('TOKEN_ENDPOINT_AUTH_METHOD', self.TOKEN_ENDPOINT_AUTH_METHOD)
+        if method:
+            return method == 'client_secret_basic'
+        methods = self.oidc_config().get('token_endpoint_auth_methods_supported', [])
+        return not methods or 'client_secret_basic' in methods
+
+    def oidc_endpoint(self):
+        return self.setting('OIDC_ENDPOINT', self.OIDC_ENDPOINT)
+
     @cache(ttl=86400)
     def oidc_config(self):
-        return self.get_json(self.OIDC_ENDPOINT +
+        return self.get_json(self.oidc_endpoint() +
                              '/.well-known/openid-configuration')
 
     @cache(ttl=86400)
@@ -159,6 +179,8 @@ class OpenIdConnectAuth(BaseOAuth2):
 
         for key in keys:
             if kid is None or kid == key.get('kid'):
+                if 'alg' not in key:
+                    key['alg'] = self.setting('JWT_ALGORITHMS', self.JWT_ALGORITHMS)[0]
                 rsakey = jwk.construct(key)
                 message, encoded_sig = id_token.rsplit('.', 1)
                 decoded_sig = base64url_decode(encoded_sig.encode('utf-8'))
@@ -184,11 +206,11 @@ class OpenIdConnectAuth(BaseOAuth2):
             claims = jwt.decode(
                 id_token,
                 rsakey.to_pem().decode('utf-8'),
-                algorithms=self.JWT_ALGORITHMS,
+                algorithms=self.setting('JWT_ALGORITHMS', self.JWT_ALGORITHMS),
                 audience=client_id,
                 issuer=self.id_token_issuer(),
                 access_token=access_token,
-                options=self.JWT_DECODE_OPTIONS,
+                options=self.setting('JWT_DECODE_OPTIONS', self.JWT_DECODE_OPTIONS),
             )
         except ExpiredSignatureError:
             raise AuthTokenError(self, 'Signature has expired')
@@ -219,11 +241,12 @@ class OpenIdConnectAuth(BaseOAuth2):
         })
 
     def get_user_details(self, response):
-        username_key = self.setting('USERNAME_KEY', default=self.USERNAME_KEY)
+        username_key = self.setting('USERNAME_KEY', self.USERNAME_KEY)
         return {
             'username': response.get(username_key),
             'email': response.get('email'),
             'fullname': response.get('name'),
             'first_name': response.get('given_name'),
             'last_name': response.get('family_name'),
+            'groups': response.get('groups'),    # not standardized but widely implemented
         }
