@@ -1,5 +1,5 @@
 from .oauth import OAuth2Test
-from .open_id_connect import OpenIdConnectTestMixin
+from .test_open_id_connect import OpenIdConnectTestMixin
 
 
 class ElixirOpenIdConnectTest(OpenIdConnectTestMixin, OAuth2Test):
